@@ -1,4 +1,5 @@
 
+
 core.error_variable <- function(df_data, varname, funcname,type="numeric") {
   
   if (!is.null(varname)) {
@@ -327,7 +328,8 @@ core.csu_asr <- function(df_data, var_age, var_cases, var_py, group_by=NULL,
                          var_age_group=NULL, missing_age = NULL,db_rate = 100000,
                          first_age = 1, last_age = 18, pop_base = "SEGI",
                          var_st_err=NULL,correction_info=FALSE, var_asr="asr", age_dropped = FALSE,
-                         pop_base_count = NULL, age_label_list = NULL, Rcan_print=FALSE) {
+                         pop_base_count = NULL, age_label_list = NULL, Rcan_print=FALSE) 
+{
   
   
   
@@ -359,6 +361,9 @@ core.csu_asr <- function(df_data, var_age, var_cases, var_py, group_by=NULL,
   setnames(dt_data, var_cases, "CSU_C")
   setnames(dt_data, var_py, "CSU_P")
 
+  #Sum cases and pop per group_by + age _group
+  dt_data <-  dt_data[,list( CSU_C=sum(CSU_C), CSU_P=sum(CSU_P)), by=c(group_by, "CSU_A")]
+
   # create index to keep order
   index_order <- c(1:nrow(dt_data))
   dt_data$index_order <- index_order
@@ -562,14 +567,13 @@ core.csu_asr <- function(df_data, var_age, var_cases, var_py, group_by=NULL,
 }
 
 
-
-core.csu_eapc <-
-  function(df_data,
+core.csu_eapc <- function(df_data,
            var_rate="asr",
            var_year="year",
            group_by= NULL,
            var_eapc="eapc", 
-					 CI_level = 0.95) {
+					 CI_level = 0.95)
+{
     
     #create fake group to have group_by optional 
     bool_dum_by <- FALSE
@@ -630,9 +634,7 @@ core.csu_eapc <-
 
 
 
-
-core.csu_ageSpecific <-
-  function(df_data,
+core.csu_ageSpecific <-function(df_data,
            var_age="age",
            var_cases="cases",
            var_py="py",
@@ -653,7 +655,7 @@ core.csu_ageSpecific <-
 					 ytitle = "Age-specific incidence rate per",
 					 label_group_by = waiver())
 
-					 {
+{
     
     
     
@@ -1162,7 +1164,7 @@ core.csu_time_trend <- function (
   # to calcul y axes breaks
   
   if (logscale) {
-    min_tick_value <- min(dt_data[CSU_smooth != 0,]$CSU_smooth)
+    min_tick_value <- min(dt_data[CSU_smooth > 0,]$CSU_smooth)
   } else {
     min_tick_value <- 0
   }
@@ -1307,4 +1309,253 @@ core.csu_time_trend <- function (
 }
 
 
+core.csu_year_extract <- function(year_list) {
+
+  temp <- gsub("[^\\d]", "", year_list, perl=TRUE)
+  if (unique(nchar(temp)) == 4 ) {
+    return(as.numeric(year_list))
+  }
+  else if (unique(nchar(temp)) == 6 ) {
+    
+      test <- all(grepl("(18|19|20)\\d{2}\\d{2}",temp))
+      if (test) {
+        return(as.numeric(gsub("(^\\d{4}).+", "\\1", temp, perl=TRUE))) 
+      }
+      else {
+        return(as.numeric(gsub(".+(\\d{4}$)", "\\1", temp, perl=TRUE))) 
+      }
+
+  }
+  else if (unique(nchar(temp)) == 8 ) {
+     
+     test <- all(grepl("(18|19|20)\\d{2}\\d{4}",temp))
+     if (test) {
+        return(as.numeric(gsub("(^\\d{4}).+", "\\1", temp, perl=TRUE))) 
+     }
+     else {
+        return(as.numeric(gsub(".+(\\d{4}$)", "\\1", temp, perl=TRUE))) 
+     }
+  }
+}
+
+
+core.csu_icd_group <- function(icd_list) {
+
+  bool_follow <- FALSE 
+  icd_first <- icd_list[1]
+  icd_long <- icd_first
+
+  code_active = as.numeric(sub(".+?(\\d+)", "\\1", icd_first))
+
+  for (code in icd_list[-1]) {
+
+    code_new = as.numeric(sub(".+?(\\d+)", "\\1", code))
+    bool_follow <- code_new == code_active + 1
+
+    if (bool_follow) {
+      icd_long <- paste0(icd_first, '-', sprintf("%02d", code_new))
+
+    }
+    else {
+      icd_long <- paste0(icd_long, ',', as.character(code))
+      icd_first <- icd_long
+    }
+    code_active <- code_new
+
+  }
+  return(icd_long)
+}
+
+
+core.csu_icd_ungroup <- function(icd_group) {
+
+  icd_group <- gsub("\\s", "", icd_group)
+  
+  icd_list <- NULL
+  ICD_reg <-"(C\\d+)([^C+]?)(.+)?"
+
+  while (nchar(icd_group)>=3) {
+
+    icd_start <- sub(ICD_reg, "\\1", icd_group)
+    icd_mark <- sub(ICD_reg, "\\2", icd_group)
+    icd_group <- sub(ICD_reg, "\\3", icd_group)
+
+
+    if (icd_mark == "-") {
+
+      code_start <- sub("C(\\d+)", "\\1", icd_start)
+      code_nchar <- nchar(code_start)
+      code_start <- as.numeric(code_start)
+      code_end <- as.numeric(sub("C?(\\d+)(.+)?", "\\1", icd_group))
+
+      for (code in code_start:code_end) {
+        icd_list <- c(icd_list, paste0("C", sprintf(paste0("%0",code_nchar,"d"), code)))
+      }
+
+      icd_group <- sub("(C?\\d+)([^C+]?)(.+)?", "\\3", icd_group) 
+    }
+    else  {
+      icd_list <- c(icd_list, sub("(C\\d+)", "\\1", icd_start))
+    }
+  }
+
+  return(icd_list)
+}
+
+
+core.csu_group_cases <- function(df_data, var_age ,group_by=NULL,var_cases = NULL,df_ICD = NULL,var_ICD=NULL,var_year = NULL, all_cancer=FALSE) {
+
+
+  label_by <- NULL
+  dt_data <- data.table(df_data)
+
+  bool_dum_by <- FALSE
+  if (is.null(group_by)) {
+      
+    dt_data$CSU_dum_by <- "dummy_by"
+    group_by <- "CSU_dum_by"
+    bool_dum_by <- TRUE
+  }
+  
+
+  if (is.null(var_cases)) {
+    var_cases <- "CSU_C"
+    dt_data[, CSU_C:= 1]
+  } else {
+    setnames(dt_data, var_cases, "CSU_C")
+  }
+
+  dt_data <- dt_data[, unique(c("CSU_C", var_age,group_by,var_ICD,var_year)), with = FALSE]
+
+  if (!is.null(var_year)) {
+    dt_data$year <-  core.csu_year_extract(dt_data[[var_year]])
+    dt_data[, (var_year) := NULL]  
+    group_by <- c(group_by, "year")
+
+  }
+
+
+  if (!is.null(df_ICD)) {
+  # merge with ICD 
+    dt_ICD <- data.table(df_ICD)
+    setkeyv(dt_ICD,c("LABEL", "ICD")) 
+    dt_ICD <- unique(dt_ICD)
+
+    #ungroup ICD code 
+    dt_table <- data.table()
+    for (row in 1:nrow(dt_ICD)) {
+
+      icd_group <- as.character(dt_ICD[row]$ICD)
+      temp <- core.csu_icd_ungroup(paste(icd_group, collapse=","))
+      temp <- data.table(ICD_ungroup = temp, ICD =icd_group )
+      
+      dt_table <- rbind(dt_table, temp)
+
+    }
+
+    dt_ICD <- merge(dt_ICD, dt_table, by="ICD")
+    dt_ICD[, ICD:=NULL]
+    dt_ICD <- unique(dt_ICD)
+
+    dt_ICD_unique <- setDT(dt_ICD)[, .N, keyby=ICD_ungroup][N>1,]  
+
+    if (nrow(dt_ICD_unique) > 0) {
+
+      dt_ICD_unique <- merge(dt_ICD_unique,dt_ICD,by="ICD_ungroup", all.x=TRUE) 
+      dt_ICD_unique <- merge(dt_ICD_unique, df_ICD, by="LABEL")
+      dt_ICD_unique <- dt_ICD_unique[,c("LABEL", "ICD") , with=FALSE]
+      cat("-----\n")
+      print(as.data.frame(dt_ICD_unique))
+      stop(paste0("There is different label with the same ICD code in the ICD file defined")) 
+
+    }
+
+    setnames(dt_ICD, "ICD_ungroup", "ICD")
+
+    dt_ICD[, ICD_group:= sapply(LABEL, function(x) {core.csu_icd_group(as.vector(dt_ICD[LABEL == x, ]$ICD))})]
+    list_ICD <- dt_ICD$ICD
+
+    dt_data$temp <- as.character(dt_data[[var_ICD]])
+    dt_data[, ICD := list_ICD[match(dt_data$temp, list_ICD)]]
+    dt_data[!is.na(ICD), temp := NA]
+    dt_data[, temp:= substr(temp, 1, 3)] 
+    dt_data[!is.na(temp), ICD:=list_ICD[match(dt_data[!is.na(temp)]$temp, list_ICD)]]
+    dt_data <- dt_data[!is.na(ICD), ]
+    dt_data <- merge(dt_data, dt_ICD, by=c("ICD"))
+    dt_data[,c("temp", var_ICD,"ICD") := list(NULL, NULL,NULL)]
+    dt_data[, LABEL := factor(LABEL)]
+    dt_data[, ICD_group := factor(ICD_group)]
+
+    label_by <- c(label_by,"LABEL")
+    group_by <- c(group_by, "ICD_group")
+
+    dt_ICD <- NULL
+
+  }
+
+  #  create age group 
+  dt_data[, age_group:= cut(get(var_age), c(seq(0, 85, 5), 150), include.lowest = TRUE, right=FALSE)]
+  dt_data[, age_group_label := as.character(age_group)]
+  dt_data[, temp1 := as.numeric(sub("\\[(\\d{1,3}),(\\d{1,3}).+", "\\1",age_group_label))]
+  dt_data[, temp2 := as.numeric(sub("\\[(\\d{1,3}),(\\d{1,3}).+", "\\2",age_group_label))]
+  dt_data[, age_group :=  ifelse(temp2 == 150 ,18,temp2/5)]
+  dt_data[, temp1 := sprintf("%02d", temp1)]
+  dt_data[, age_group_label := ifelse(temp2 == 150, paste0(temp1,"+"), paste0(temp1,"-",  sprintf("%02d", temp2-1)))] 
+  dt_data[is.na(age_group), age_group :=  19]
+  dt_data[age_group == 19 , age_group_label :=  "Unknown"]
+  dt_data[,c("temp1","temp2", var_age) := list(NULL, NULL, NULL)]
+
+  dt_data <-  dt_data[,list(CSU_C = sum(CSU_C)),by=eval(colnames(dt_data)[!colnames(dt_data) %in% c("CSU_C")])]
+
+  label_by  <- c(label_by, c("age_group_label"))
+  group_by  <- c(group_by, "age_group")
+
+  dt_CJ = dt_data[, do.call(CJ, c(.SD, unique=TRUE)), .SDcols=group_by]
+
+  ##add ICD group label (but #dad is pink)
+  if (!is.null(df_ICD)) {
+    dt_temp <- unique(dt_data[, c("ICD_group","LABEL"), with=FALSE])
+    dt_CJ <- merge(dt_CJ, dt_temp, by="ICD_group", all.x=TRUE)
+  }
+
+  ##add age group label
+  dt_temp <- unique(dt_data[, c("age_group","age_group_label"), with=FALSE])
+  dt_CJ <- merge(dt_CJ, dt_temp, by="age_group", all.x=TRUE)
+
+
+  dt_data <- merge(dt_CJ, dt_data,by=colnames(dt_CJ), all.x=TRUE)[, CSU_C := ifelse(is.na(CSU_C),0, CSU_C )]
+
+
+  keep_by <- group_by[!group_by %in% c("year", "age_group", "ICD_group")]
+
+  setkeyv(dt_data, keep_by)
+  dt_data[ , group_id := .GRP, by = key(dt_data)]
+  dt_data[,temp := sum(CSU_C),by=group_id]
+  dt_data <- dt_data[temp>0,]
+  dt_data[, c("temp","group_id") := NULL]
+  setkeyv(dt_data, group_by)
+
+
+
+  if (var_cases=="CSU_C") {
+    setnames(dt_data, "CSU_C","cases")
+    var_cases <- NULL
+  }
+  else {
+    setnames(dt_data, "CSU_C",var_cases)
+  }
+
+  if (bool_dum_by) {
+    dt_data$CSU_dum_by <- NULL
+  }
+
+  dt_data <- as.data.frame(dt_data)
+
+
+  return (dt_data)
+}
+
+
+
+   
 
